/***************************************************************************/
/*                                                                         */
/*                              CalcBias3()                                */
/*                                                                         */
/*      Function to calculate bias function B3(2, M, r, ) =  ratio of     */
/*      Allan variance with distributed dead times to Allan variance       */
/*      with all dead time at end as a function of M=# averages,           */
/*      r=T/=dead time ratio and =power law noise exponent of y().     */
/*	  	r=1 for no dead time.  B1=1 for N=2.                               */
/*                                                                         */
/*  	References:    (1) NBS Technical Note 375, J.A. Barnes, "Tables    */
/*						   of Bias Functions B1 and B2 ...", Jan. 1969.    */
/*					   (2) NIST Technical Note 1337, TN-296 to TN-335,     */
/*						   J.A. Barnes and D.W. Allan, "Variances Based    */
/*						   on Data with Dead Time Between the              */
/*                         Measurements".                                  */
/*	            	   (3) J.A. Barnes, "The Analysis of Time and          */
/*						   Frequency Data", Austron, Inc, Dec. 1991.       */
/*                                                                         */
/*  	Parameters:    int m   = # averages (tested for 1 to 1024)         */
/*				  	             OK for 1 to INT_MAX, large n takes longer */
/*					   float r = dead time ratio (tested for.01 to 8192)   */
/*                	             Bounded to 1e-4 < r < 1e6 to avoid O/F    */
/*					   float mu= noise type (tested for -2 to +2)          */
/*                	             Get -1 error code for mu < -2 or mu > 2   */
/*                                                                         */
/*		Return:		   float   = B3 bias function value,                   */
/*                               or -1 if error                            */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/24/91   Drafted & debugged                                  */
/*			12/31/91   Renamed                                             */
/*			01/06/91   Added error trap for m=0                            */
/*			01/18/92   Edited title block                                  */
/*			02/03/96   Modified for use as Win 3.1 DLL                     */
/*			01/01/98   Changes for MS VC++ compatibility & warnings        */
/*                                                                         */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

float __declspec(dllexport) FAR PASCAL CalcBias3(int m, float r, float mu)
{
	double f;                                                    /* term f */
	double s;                                            /* summation term */
	int i;                                              /* summation index */

	if(m==0)
	{
		return(-1.0);                                        /* error code */
	}
	m=abs(m);                                   /* avoid negative # points */
	r=(float)fabs(r);                              /* avoid negative ratio */

	if((mu<-2)||(mu>2))                               /* avoid <-2 or >2 */
	{
		return(-1.0);                                        /* error code */
	}                                                            /* end if */

	if((r==1)||(m==1)||(mu==2))   /* no need to calc if r=1 or m=1 or mu=2 */
	{
		return(1.0);
	}                                                            /* end if */

	if(mu==-2)                                 /* no need to calc if mu=-2 */
	{
		return(float)(m);
	}                                                            /* end if */

	if(mu==0)                                             /* check for =0 */
	{
		mu+=(float)1e-6;                           /* to avoid singularity */
	}                                                            /* end if */

	if(r<.0001)                                      /* check for r near 0 */
	{
		r=(float)1e-4;                             /* to avoid singularity */
	}                                                            /* end if */

	if(r>1e6)                                    /* check for very large r */
	{
		r=1e6;                                        /* to avoid overflow */
	}                                                            /* end if */

	s=0;                                         /* initialize sum to zero */

	for(i=1; i<m; i++)                                     /* do summation */
	{
		f=2*pow(i*r, mu+2) - pow((i*r)+1, mu+2) - pow(fabs((i*r)-1), mu+2);
		f*=2;
		f-=2*pow((m+i)*r, mu+2) - pow(((m+i)*r)+1, mu+2)
		  - pow(fabs(((m+i)*r)-1), mu+2);
		f-=2*pow((m-i)*r, mu+2) - pow(((m-i)*r)+1, mu+2)
		  - pow(fabs(((m-i)*r)-1), mu+2);
		f*=(m-i);
		s-=f;
	}                                                           /* end for */

	s+=m*(2*pow(m*r, mu+2) - pow((m*r)+1, mu+2) - pow(fabs((m*r)-1), mu+2));
	s+=2*m;
	f=2*pow(r, mu+2) - pow(r+1, mu+2) - pow(fabs(r-1), mu+2);
	f+=2;

	return(float)(s/(f*pow(m, mu+2)));
}                                                       /* end CalcBias3() */

/***************************************************************************/
