/***************************************************************************/
/*                                                                         */
/*								CalcInvChiSqr()                            */
/*                                                                         */
/*		This function uses an iterative process to calculate the           */
/*		value of chi squared associated with a given probability for       */
/*		a certain # of degrees of freedom.  Calls CalcChiSqrProb().        */
/*	                                                                       */
/*	    Parameters:	   float p	= chi squared probability value            */
/*					   float df	= # of degrees of freedom                  */
/*	                                                                       */
/*		Return:	       float 	= chi square value,                        */
/*                                or -1 if error                           */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/30/91   Created                                             */
/*			12/31/91   Renamed                                             */
/*			01/05/92   Added error trap                                    */
/*			01/18/92   Edited title block                                  */
/*			02/03/96   Modified for use as Win 3.1 DLL                     */
/*			01/01/98   Changes for MS VC++ compatibility & warnings        */
/*																		   */
/* (c) Copyright 1991-8  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

float __declspec(dllexport) FAR PASCAL CalcInvChiSqr(float p, float df)
{
	double x, prob;                               /* chi-squared variables */
	double sign, div;           /* chi-squared iteration control variables */

	if(p<=0.00001)                  /* check for small, zero or negative p */
	{                                   /* no/slow convergence for small p */
		return(-1.0);                                        /* error code */
	}

	x = df+(0.5-p)*df*0.5;                  /* start value for chi squared */
	prob = CalcChiSqrProb((float)x, df);
	div = 0.1;
	while(fabs((prob-p)/p)>0.0001)                   /* accuracy criterion */
	{
		if(prob-p>0)                                 /* save sign of error */
		{
			sign=1.0;
		}
		else
		{
			sign=-1.0;
		}
		x += df*(prob-p)/div;                       /* iteration increment */
		if(x>0)                                 /* make sure argument is + */
		{
			prob = CalcChiSqrProb((float)x, df);
		}
		else                    /* otherwise restore it & reduce increment */
		{
			x -= df*(prob-p)/div;
			div*=2;
		}
		if(((prob-p)/sign)<0)                /* did sign of error reverse? */
		{
			div*=2;                          /* reduce increment if it did */
		}
	}                                                         /* end while */

	return(float)x;                                 /* end CalcInvChiSqr() */
}
/***************************************************************************/
