/*****************************************************************************/
/*                                                                           */
/*								CalcInvChiSqr()                              */
/*                                                                           */
/*		1st function calculates closed-form approximation to inverse chi     */
/*      square for large # degrees of freedom (OK for df>10)                 */
/*                                                                           */
/*      References:	(1) C. Greenhall/JPL via e-mail 12/06/96                 */
/*                  (2) Abramowitz & Stegun, Handbook of Mathematical        */
/* 						Functions, Sections 26.2.22 & 26.4.17                */
/*                                                                           */
/*		2nd function uses a WJR iterative process to calculate the           */
/*		value of chi squared associated with a given probability for         */
/*		a certain # of degrees of freedom.  Calls CalcChiSqrProb().          */
/*	                                                                         */
/*	    Parameters:	   float p	= chi squared probability value              */
/*					   float df	= # of degrees of freedom (not necessarily   */
/*								  an integer)                                */
/*	                                                                         */
/*		Return:	       float 	= chi square value,                          */
/*                                or -1 if error                             */
/*                                                                           */
/*		Revision record:                                                     */
/*			02/03/97   Adapted from CalcInvChiSqr() of FrequenC Library      */
/*			02/04/97   Added closed-form function for large df.  Faster,     */
/*					   and avoids possible convergence problems.             */
/*			02/05/97   Fixed problem with wrong tail for new large df calc   */
/*																		     */
/*****************************************************************************/

#include "frequenc.h"

float __declspec(dllexport) FAR PASCAL CalcInvChiSqr(float p, float df)
{
	// Local variables
	double x;                     	// Chi-squared value
	double prob;                    // Degrees of freedom
	double sign;                    // Iteration control sign
	double div;           			// Iteration control divisor

	// Closed-form variables
	double a0=2.30753;           	// Coefficients from A&S 26.2.22
	double a1=0.27601;              // "
	double b1=0.99229;              // "
	double b2=0.04481;              // "
	double b;
	double n;
	double p1;
	double s;
	double t;
	double y;

	if(p<=0.00001)                    /* check for small, zero or negative p */
	{                                     /* no/slow convergence for small p */
		return(-1.0);                                          /* error code */
	}

	// Use closed-form approximation for df > 100
	if(df>100)
	{
		// Use opposite tail of distribution
		p=1-p;

		// Calc approximation to inverse chi sqr for large df
		p1=min(p, 1-p);
		t=sqrt(-2.0*log(p1));                   	// A&S 26.2.22
		n=t-((a0+(a1*t))/(1+(t*(b1+(t*b2)))));      // "
		b=2.0/(9.0*df);                             // A&S 26.4.17

		if((p-0.5)<0) s=-1.0;                       // Greenhall revision
		else if((p-0.5)>0) s=1.0;                   // 1 function for
		else s=0.0;                                 // both tails

		y=1-b+(s*n*sqrt(b));                       	// A&S 26.4.17
		x=df*pow(y, 3.0);                           // "

		return (float)x;
	}

	// Iterative solution for df<=100
	x = df+(0.5-p)*df*0.5;                    /* start value for chi squared */
	prob = CalcChiSqrProb(x,df);
	div = 0.1;

	while(fabs((prob-p)/p)>0.0001)                     /* accuracy criterion */
	{
		if(prob-p>0)                                   /* save sign of error */
		{
			sign=1.0;
		}
		else
		{
			sign=-1.0;
		}
		x += df*(prob-p)/div;                         /* iteration increment */
		if(x>0)                                   /* make sure argument is + */
		{
			prob = CalcChiSqrProb(x, df);
		}
		else                      /* otherwise restore it & reduce increment */
		{
			x -= df*(prob-p)/div;
			div*=2;
		}
		if(((prob-p)/sign)<0)                  /* did sign of error reverse? */
		{
			div*=2;                            /* reduce increment if it did */
		}
	}
	return (float)x;
}

/*****************************************************************************/
