/*****************************************************************************/
/*                                                                           */
/*                            CalcPhaseModSigma()                            */
/*                                                                           */
/*      Function to calculate the modified Allan variance for phase data,    */
/*      possibly with gaps.  A faster function, CalcFastModSigma(), can      */
/*      be used if there are no gaps.                                        */
/*                                                                           */
/*      Parameters:     F_TYPE x[]     = phase data                          */
/*                                       x[0] = # data phase points          */
/*                                       x[1] = analysis start               */
/*                                       x[2] = analysis end                 */
/*                      F_TYPE *p_sig  = pointer to mod Allan deviation      */
/*                      F_TYPE tau     = analysis averaging time = m*tau_0   */
/*                      int m          = averaging factor = tau/tau_0        */
/*						BOOL *bAbort   = pointer to abort flag               */                                                                     
/*						BOOL bProgress = progress indicator flag             */                                                                     
/*						void *ProgressFunction()                             */
/*									   = pointer to progress indicator       */
/*									 	 function with args int nPercentDone */
/*										 and char *szMessage                 */
/*                                                                           */
/*      Return:         int            = # analysis points,                  */
/*                                       or -1 if error                      */
/*                                                                           */
/*      Notes:          Embedded zero phase value treated as gap in data     */
/*                      Phase data is not averaged before calling this       */
/*                      function.  The data consists of num phase samples    */
/*                      at interval tau_0.  Denominator term (num-3m+1)      */
/*                      must be >0.  This sets max m=(int)(num/3),           */
/*                      where num=# phase points before averaging.           */
/*                      m must be >0 and tau must be >0.                     */
/*                      Use faster CalcFastModSigma() if no gaps in data.    */
/*                                                                           */
/*						The progress function pointer can be NULL, but is    */
/*                      intended as a way to display the % done and a        */
/*                      message about the calculation.                       */
/*                                                                           */
/*                      The abort flag provides a way to abort the calc.     */
/*                      It must exist but does not have to be used.          */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/16/90    Created                                              */
/*          12/31/91    Renamed                                              */
/*          01/18/92    Edited title block                                   */
/*          01/19/92    Added error traps                                    */
/*          02/03/96    Modified for use as Win 3.1 DLL                      */
/*          06/15/02    Copied here for revision.  Added gap flag.           */
/*                      Fixed error for small AF and large sets of low       */
/*                      resolution noise where sum_i can be zero w/o gap     */
/*          01/28/03    Added progress indicator & argument                  */
/*			03/08/03    Brought into Ver 2.0 of FrequenC lib                 */
/*						Added progress message function pointer to arg list  */
/*			03/09/03	Added bAbort pointer to arg list                     */
/*                                                                           */
/* (c) Copyright 1990-2003 Hamilton Technical Services  All Rights Reserved  */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

int __declspec(dllexport) FAR PASCAL CalcPhaseModSigma(F_TYPE x[],
	F_TYPE *p_sig, F_TYPE tau, int m, BOOL *bAbort, BOOL bProgress,
	void *ProgressFunction(int nPercentDone, char *szMessage))
{
    // Local variables
	double sum_i, sum_j;              /* summing variables to calc mod sigma */
    float fm;                                          /* float version of m */
    int i;                                   /* index for inner summing loop */
    int j;                                   /* index for outer summing loop */
    int n_sum=0;                           /* # points in outer summing loop */
    int num = (int)(*(x+2)-*(x+1)+1);    /* # phase data points for analysis */
    int gap;                                                     /* gap flag */
    int nTic;                            /* Increment for progress indicator */
    int nLast_j=0;                                /* Previous value of index */

    // Check args
	if( (m<1) || (num<3*m) || (tau<=0) )
    {
        return(-1);                                                 /* error */
    }                                                              /* end if */

    // Initialize increment for progress report
    nTic=(int)ceil(num/100);

    // Start
    if(bProgress)
    {
        // Initialize progress display
        ProgressFunction(0,
            "Calculating Mod/Time Sigma (with gaps)"); 
    }                                                             /* end if */

    // Initialize outer loop
	fm=(float)m;
    sum_j=0;
    
	// Calc mod sigma
	for(j=1; j<=num-(3*m)+1; j++)                              /* outer loop */
            /* j goes from 1 to N-3m+1 where N=# phase points & m=avg factor */
                                            /* regardless of analysis limits */
    {
        sum_i=0;
        gap=1;
        for(i=j+(int)(*(x+1))-1; i<=m+j-1+(*(x+1))-1; i++)     /* inner loop */
                    /* i is the actual phase data index;  it includes a term */
                      /* *(x+1)-1 to account for the analysis starting point */
                   /* the data array offset +2 is included in the code below */
        {
            if(    ( i==j+(*(x+1))-1     && *(x+i+2+m) && *(x+i+2+(2*m)) )
               ||  ( *(x+i+2)            && *(x+i+2+m) && *(x+i+2+(2*m)) )
               ||  ( i==m+j-1+(*(x+1))-1 && *(x+i+2+m) && *(x+i)       ) )
                            /* skip if any imbedded phase data value is zero */
            {
                sum_i += *(x+i+2+(2*m)) - (2*(*(x+i+2+m))) + *(x+i+2);
                gap=0;                                          /* not a gap */
            }                                                      /* end if */
        }                                                         /* end for */
        
        if(!gap) 
        {
            sum_j += SQR(sum_i);
            n_sum++;
        }                                                          /* end if */

        // Display progress
        // Update when index has changed by at least tic amount
        if(((j-nLast_j)>=nTic) && (bProgress))
        {
            ProgressFunction((int)(0.5+100.0*j/(num-3*m)),
                "Calculating Mod/Time Sigma (with gaps)");
            nLast_j=j;
        }                                                         /* end if */

		// Check for abort
        if(*bAbort)
        {
            break;
        }
    }                                                             /* end for */

    // Done

    // Clear progress display
    ProgressFunction(0, "");

    // Return results
    if(n_sum>0)                                     /* make sure no div by 0 */
    {
        *p_sig = sqrt((sum_j)/(2.0*tau*tau*fm*fm*n_sum));
        return(n_sum);                                  /* # analysis points */
    }
    else
    {
        return(-1);                                                 /* error */
    }
}                                                 /* end CalcPhaseModSigma() */

/*****************************************************************************/

