/***************************************************************************/
/*                                                                         */
/*                       CalcPhaseOverlapSigma()                           */
/*                                                                         */
/*      Function to calculate Allan variance using overlapping samples     */
/*      of phase data                                                      */
/*                                                                         */
/*      Parameters:     F_TYPE x[]    = phase data array                   */
/*                                      x[0] =  # phase data points        */
/*                                      x[1] = analysis start              */
/*                                      x[2] = analysis end                */
/*                      F_TYPE *p_sig = pointer to Allan deviation         */
/*                      F_TYPE tau    = averaging time of phase data       */
/*                      int m         = averaging factor                   */
/*                                                                         */
/*      Return:         int           = # analysis points,                 */
/*                                      or -1 if error                     */
/*                                                                         */
/*      Notes:          Embedded zeros treated as gaps in phase data.      */
/*                      Must have at least 1 analysis point; this          */
/*                      requires at least 3 adjacent non-gap phase data    */
/*                      points.  Averaging time for sigma calculation is   */
/*                      m times the tau parameter of the phase data.       */
/*                      m must be >0 , tau must be >0, and the # non-gap   */
/*                      phase data points N must be > 2m.                  */
/*                                                                         */
/*      Revision record:                                                   */
/*          12/16/90    Created                                            */
/*          12/31/91    Renamed                                            */
/*          01/18/92    Edited title block                                 */
/*          01/19/92    Added error traps                                  */
/*          02/03/96    Modified for use as Win 3.1 DLL                    */
/*                                                                         */
/* (c) Copyright 1990-6  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

int __declspec(dllexport) CalcPhaseOverlapSigma(F_TYPE x[], F_TYPE *p_sig,
	F_TYPE tau, int m)
{
    double sum=0;                        /* summing variable to calc sigma */
    float fm=m;                                      /* float version of m */
    int i;                                          /* index for summation */
    int num=0;                                        /* # analysis points */
    int n;                                      /* # frequency data points */

    n=*(x+2)-*(x+1)+1;                        /* total # phase data points */

    if( (m<1) || (n<2*m) || (tau<=0) )
    {
        return(-1);                                               /* error */
    }

    for( i=*(x+1)+2; i<=n-2*m+2; i++)                      /* summing loop */
    /* i basically goes from 1 to N-2m where N is # phase data points and  */
    /* m is averaging factor.  Term *(x+1) is the analysis start limit (1  */
    /* if all data used) & the 2 accounts for the data array offset        */
    {
        if(    ( i==*(x+1)+2 && *(x+i+m) && *(x+i+(2*m)) )
           ||  ( *(x+i)      && *(x+i+m) && *(x+i+(2*m)) )
           ||  ( i==n-2*m+2  && *(x+i+m) && *(x+i)       ) )
                          /* skip if any imbedded phase data value is zero */
        {
            sum += SQR ( *(x+i+(2*m)) - (2 * (*(x+i+m))) + *(x+i) );
            num++;
        }                                                        /* end if */
    }
    if(num>0)
    {
        *p_sig=sqrt( sum / (2.0*tau*tau*fm*fm*num));
        return(num);                                  /* # analysis points */
    }
    else
    {
        return(-1);                                              /* error */
    }
}                                           /* end CalcPhaseOverlapSigma() */

/***************************************************************************/
