/***************************************************************************/
/*                                                                         */
/*					         CalcQuadraticDrift()                          */
/*                                                                         */
/*		Function to calculate least-squares quadratic fit to phase data    */
/* 		x(t) = a + bt + ct                                                */
/*                                                                         */
/* 		Reference:	    J.A. Barnes, "The Measurement of Linear Frequency  */
/*                      Drift in Oscillators", Proc. 15th Annual PTTI      */
/*                      Meeting, 1983, pp. 551-582.                        */
/*                                                                         */
/*		Parameters: 	F_TYPE x[]  = phase data array                     */
/*                                    x[0] = # phase data points           */
/*                                    x[1] = analysis start                */
/*                                    x[2] = analysis end                  */
/*						F_TYPE tau  = time interval for phase data         */
/*                      F_TYPE *p_a = pointer to phase intercept           */
/*						F_TYPE *p_b = pointer to frequency intercept       */
/*						F_TYPE *p_c = pointer to frequency slope           */
/*						F_TYPE *p_v = pointer to fit variance              */
/*                                                                         */
/*		Return:         int         = # analysis points                    */
/*									  or -1 if error                       */
/*                                                                         */
/*      Notes:          x(t) , t & a in sec, b in sec/sec, c in sec/sec.  */
/* 		                Frequency drift (slope) = 2c; intercept = b.       */
/* 		                t is time interval of equally-spaced phase data.   */
/* 		                t goes from 1 to n with no missing points.         */
/*                      Calls FrequenC functions CountGaps() and           */
/*					    FillGaps() as needed.  To preserve gaps, save      */
/*					    data before calling this function.                 */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/29/91	Created                                            */
/*			12/31/91	Renamed                                            */
/*          01/18/92    Edited title block                                 */
/*			01/19/92	Added error traps                                  */
/*			01/22/92	Added error trap for tau<=0                        */
/*			11/04/92	Changed gap handling -- was NG                     */
/*			02/03/96	Modified for use as Win 3.1 DLL                    */
/*                                                                         */
/* (c) Copyright 1991-6  Hamilton Technical Services  All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"                              /* FrequenC header file */

int F__declspec(dllexport) CalcQuadraticDrift(F_TYPE x[], F_TYPE tau,
	F_TYPE *p_a, F_TYPE *p_b, F_TYPE *p_c, F_TYPE *p_v)
{
	double aa,bb,cc,dd,ee,ff,gg;                              /* fit terms */
	double sx,sxx,snx,snnx;                                  /* summations */
	int n=x[2]-x[1]+1;                                         /* # points */
	int i;                                                   /* data index */
	int j=1;                                            /* summation index */

	/* check arguments */
	if(tau<=0)                                    /* tau negative or zero? */
	{
		return(-1);                                               /* error */
	}

	if(n<3)                   /* need at least 3 points to calc parameters */
	{
		return(-1);                                               /* error */
	}

	/* fill gaps if necessary */
	if( CountGaps(x, PHASE) )                                 /* any gaps? */
	{
		FillGaps(x, PHASE);                     /* fill gaps in phase data */
	}

	/* initialize sums */
	sx = sxx = snx = snnx = 0;

	/* calculate sums for quadratic fit */
	for(i=x[1]; i<=x[2]; i++)
	{
		sx  += x[i+2];
		sxx += SQR(x[i+2]);
		snx += x[i+2]*j;
		snnx+= x[i+2]*j*j;
		j++;
	}                                                           /* end for */

	/* calculate terms for quadratic fit */
	aa = 3*((3*n*(n+1))+2);
	bb = -18*((2*n)+1);
	cc = 30;
	dd = (12*((2*n)+1)*((8*n)+11))/((n+1)*(n+2));
	ee = -180/(n+2);
	ff = 180/((n+1)*(n+2));
	gg = n*(n-1)*(n-2);

	/* calculate coefficients for quadratic fit */
	*p_a = ((aa*sx)+(tau*bb*snx)+(tau*tau*cc*snnx))/(gg        );
	*p_b = ((bb*sx)+(tau*dd*snx)+(tau*tau*ee*snnx))/(gg*tau    );
	*p_c = ((cc*sx)+(tau*ee*snx)+(tau*tau*ff*snnx))/(gg*tau*tau);
	*p_v=0;                    /* fit variance 0 if only 3 analysis points */

	if(n>=4)       /* need at least 4 analysis points to calc fit variance */
	{
		*p_v = (sxx-(*p_a*sx)-(tau**p_b*snx)-(tau*tau**p_c*snnx))/(n-3);
	}

	return(n);
}                                              /* end CalcQuadraticDrift() */

/***************************************************************************/
