/***************************************************************************/
/*                                                                         */
/*							  FindMedian                                   */
/*                                                                         */
/*		Function to find median of array of phase or frequency data        */
/*                                                                         */
/*		Parameters:	    F_TYPE z[]    = phase or frequency data            */
/*                                      z[0] = # data points               */
/*			                            z[1] = analysis start              */
/*                                      z[2] = analysis end                */
/*			            BOOL   datype = data type: 0=phase, 1=frequency    */
/*                                                                         */
/*	    Notes:	        All zeros treated as gaps in frequency data.   	   */
/*			            Embedded zeros treated as gaps in phase data.      */
/*						This function first fills gaps in the data         */
/*						with interpolated values by calling FillGaps().    */
/*						It then sorts the data into assending order,       */
/*						using qsort(), to find the median value.           */
/*						The median is the center value for an odd # of     */
/*						data points, and the average of the two center     */
/*						values for an even # of data points.               */
/*						To preserve the original data , save it before     */
/*                      calling this function.                             */
/*                                                                         */
/*      Return:         F_TYPE            = median of data points          */
/*                                                                         */
/*		Revision record:                                                   */
/*			12/12/92	Created                                            */
/*			12/25/92	Changed x[] to z[] in definition; edited comments  */
/*                                                                         */
/* (c) Copyright 1992   Hamilton Technical Services   All Rights Reserved  */
/*                                                                         */
/***************************************************************************/

#include "frequenc.h"

/* declare comparison function - not exported */
static int median_compare(const void *a, const void *b);

F_TYPE __declspec(dllexport) FAR PASCAL FindMedian(F_TYPE z[], BOOL datype)
{
	int i;                                                    /* 1st index */
	int j=0;                                                  /* 2nd index */
	int start=(int)z[1];                                 /* analysis start */
	int end=(int)z[2];                                     /* analysis end */
	int num;                                              /* # data points */

	/* fill gaps in data */
	FillGaps(z, datype);

	/* eliminate non-analysis points */
	for(i=start+2; i<=end+2; i++)
	{
		z[j++]=z[i];
	}                                                           /* end for */
	num=end-start+1;

	/* sort data into numerical order */
	qsort((void *) z, (size_t) num, sizeof(F_TYPE), median_compare);

	/* find & return median */
	if(num%2)                                               /* is num odd? */
	{
		return(z[num/2]);                                  /* middle value */
	}                                                            /* end if */
	else
	{                                                         /* n is even */
		return((z[(num/2)-1]+z[num/2])/2.0); /* average of 2 middle values */
	}                                                          /* end else */
}                                                      /* end FindMedian() */

/***************************************************************************/

static int median_compare(const void *a, const void *b)
{
	F_TYPE x =  (*(F_TYPE *) a) - (*(F_TYPE *) b) ;
	return (x>0) ? 1 : (x<0) ? -1 : 0;
}

/***************************************************************************/
