/*****************************************************************************/
/*                                                                           */
/*							    FindMedian()                                 */
/*                                                                           */
/*		Function to find median of array of phase or frequency data          */
/*                                                                           */
/*		Parameters:	    F_TYPE z[]    = phase or frequency data              */
/*                                      z[0] = # data points                 */
/*			                            z[1] = analysis start                */
/*                                      z[2] = analysis end                  */
/*			            BOOL   datype = data type: 0=phase, 1=frequency      */
/*						F_TYPE *median= pointer to median value of data      */
/*                                                                           */
/*	    Notes:	        All zeros treated as gaps in frequency data.         */
/*			            Embedded zeros treated as gaps in phase data.        */
/*						This function first fills gaps in the data           */
/*						with interpolated values by calling fill().          */
/*						It then sorts the data into assending order,         */
/*						using qsort(), to find the median value.             */
/*						The median is the center value for an odd # of       */
/*						data points, and the average of the two center       */
/*						values for an even # of data points.                 */
/*                                                                           */
/*      Return:         int            = error flag: 0=OK, 1=NG              */
/*                                                                           */
/*		Revision record:                                                     */
/*			05/03/97	Adapted from FindMedian() of STABLE_A.C              */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/* (c) Copyright 1992-8  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

/* declare comparison function - not exported */
static int median_compare(const void *a, const void *b);

int __declspec(dllexport) FAR PASCAL FindMedian(F_TYPE z[], BOOL datype,
	F_TYPE *median)
{
	int i;                                                      /* 1st index */
	int j=0;                                                    /* 2nd index */
	int start=(int)z[1];                                   /* analysis start */
	int end=(int)z[2];                                       /* analysis end */
	int num=(int)z[0];                                      /* # data points */
	F_TYPE *temp;                 /* pointer to temporary data storage array */
	F_TYPE med;                                              /* median value */

	// save data
	if((temp=(F_TYPE *)malloc((num+3)*sizeof(F_TYPE)))==NULL)
									/* allocate temporary data storage space */
	{
		return(1);                                           /* error return */
	}                                                              /* end if */

	for(i=0; i<num+3; i++)
	{
		temp[i]=z[i];                                           /* save data */
	}                                                             /* end for */

	/* Fill any gaps in data */
	FillGaps(z, datype);

	/* eliminate non-analysis points */
	for(i=start+2; i<=end+2; i++)
	{
		z[j]=z[i];
		j++;
	}                                                             /* end for */
	num=end-start+1;

	/* sort data into numerical order */
	qsort((void *) z, (size_t) num, sizeof(F_TYPE), median_compare);

	/* find & return median */
	if(num%2)                                                 /* is num odd? */
	{
		med=z[num/2];                                        /* middle value */
	}                                                              /* end if */
	else
	{                                                           /* n is even */
		med=(z[(num/2)-1]+z[num/2])/2.0;       /* average of 2 middle values */
	}                                                            /* end else */

	// restore data
	for(i=0; i<num+3; i++)
	{
		z[i]=temp[i];                                        /* restore data */
	}                                                             /* end for */
	free(temp);                                         /* deallocate temp[] */

	*median=med;                       /* set median to med = median of data */
	return(0);                                              /* normal return */

}                                                        /* end FindMedian() */

/*****************************************************************************/

static int median_compare(const void *a, const void *b)
{
	F_TYPE x =  (*(F_TYPE *) a) - (*(F_TYPE *) b) ;
	return (x>0) ? 1 : (x<0) ? -1 : 0;
}

/*****************************************************************************/
