/*****************************************************************************/
/*                                                                           */
/*                             ModTotvarBias()                               */
/*                                                                           */
/*      Function to calculate the Mod Totvar bias function                   */
/*                                                                           */
/*      Parameters:     int   nAlpha = noise type (-2 to +2)                 */
/*																			 */
/*                                                                           */
/*      Return:         float fBias  = Mod Totvar bias factor                */
/*                                     or -1 if error                        */
/*                                                                           */
/*      Alpha     Noise     MTOT/MVAR                                        */
/*       +2        W PM        1.06                                          */
/*       +1        F PM        1.17                                          */
/*        0        W FM        1.27                                          */
/*       -1        F FM        1.30                                          */
/*       -2       RW FM        1.31                                          */
/*                                                                           */
/*      The Mod Totvar bias factor is the ratio of the expected value of     */
/*		MTOT to MVAR.  To correct a calculated Mod Totdev value for a        */
/*		certain noise type, divide the uncorrected Mod Totdev value by the   */
/*      square root of the bias factor returned by this function.            */
/*                                                                           */
/*		Notes:			(1) This function applies to -2 >= alpha >= 2        */
/*		 				    (W PM, F PM, W FM, F FM, RW FM noise)            */
/*						(2) This function applies for all T/tau              */
/*                                                                           */
/* 		Reference:		E-mail from D.A. Howe/NIST,  3/13/00                 */
/*                                                                           */
/*      Revision record:                                                     */
/*          03/25/00    Created                                              */
/*                                                                           */
/* (c) Copyright 2000   Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float __declspec(dllexport) ModTotvarBias(int nAlpha)
{
	// Local variable
	float fBias;

	// Verify parameter
	if(nAlpha<-2 || nAlpha>2)
	{
		return (float) -1.0; // Error
	}

	// Sort by noise type
	switch(nAlpha)
	{
		case -2: // RW FM
		{
			fBias=(float) 1.31;
		}
		break;

		case -1: // F FM
		{
			fBias=(float) 1.30;
		}
		break;

		case -0: // W FM
		{
			fBias=(float) 1.27;
		}
		break;

		case 1: // F PM
		{
			fBias=(float) 1.17;
		}
		break;

		case 2: // W PM
		{
			fBias=(float) 1.06;
		}
		break;
	}

	return (fBias);
}

/*****************************************************************************/
