/*****************************************************************************/
/*                                                                           */
/* 		                       ModTotvarEDF()                                */
/*                                                                           */
/* 	Function to determine the estimated # of chi-squared degrees of freedom  */
/*  for the modified total variance (MTOT) of a power law noise process.     */
/*                                                                           */
/*  Params:			int   nAlpha = alpha                                     */
/*                  float fRatio = m = T/tau                                 */
/*                                                                           */
/*	Return:         float edf = estimated degrees of freedom                 */
/*								or -1 if error                               */
/*                                                                           */
/*  The edf is modeled by the expression b(T/tau) - c, where:                */
/*                                                                           */
/*  Alpha    Noise     b      c                                              */
/*   2        W PM    1.90   2.10                                            */
/*   1        F PM    1.20   1.40                                            */
/*   0        W FM    1.10   1.20                                            */
/*  -1        F FM    0.85   0.50                                            */
/*  -2       RW FM    0.75   0.31                                            */
/*                                                                           */
/*                                                                           */
/*	Notes:			(1) This function applies only to -2 >= alpha >= 2       */
/*						(W PM, F PM, W FM, F FM and RW FM noise)             */
/*					(2) This function applies only for T/tau > 8.0           */
/*					(3) T is defined as tau0 * # phase data points           */
/*                                                                           */
/* 	Reference:		E-Mail from D.A. Howe/NIST, 3/13/00                      */
/*                                                                           */
/*  Revision record:                                                         */
/*      03/25/00    Created                                                  */
/*                                                                           */
/* (c) Copyright 2000  Hamilton Technical Services   All Rights Reserved     */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float __declspec(dllexport) ModTotvarEDF(int nAlpha, float fRatio)
{
	// Local variables
	float fEDF;

	#if(0) // For testing
	TRACE_INT(nAlpha);
	TRACE_EXP(fRatio);
	#endif

	// Verify parameters
	if(fRatio<8.0)
	{
		return -1.0; // Error
	}

	if(nAlpha<-2 || nAlpha>2)
	{
		return -1.0; // Error
	}

	switch(nAlpha)
	{
		case 2:	// W PM noise
		{
			fEDF=(float)(1.90*fRatio - 2.10);
		}
		break;

		case 1:	// F PM noise
		{
			fEDF=(float)(1.20*fRatio - 1.40);
		}
		break;

		case 0:	// W FM noise
		{
			fEDF=(float)(1.10*fRatio - 1.20);
		}
		break;

		case -1: // F FM noise
		{
			fEDF=(float)(0.85*fRatio - 0.50);
		}
		break;

		case -2: // RW FM noise
		{
			fEDF=(float)(0.75*fRatio - 0.31);
		}
		break;
	}

	#if(0) // For testing
	TRACE_EXP(fEDF);
	#endif

	return fEDF;

}

/*****************************************************************************/
