/*****************************************************************************/
/*                                                                           */
/*                                 NewTheo1()                                */
/*                                                                           */
/*      Function to calculate New Tho1 deviation                            */
/*		Different function signature as original Theo1()                     */
/*                                                                           */
/*		NOTICE:  CANNOT PUT IN DLL - HAS NG CALL TO AutoTheo1Bias            */
/*                                                                           */
/*		Removes a computed bias between Tho1 and Avar to create an          */
/*		unbiased version of Tho1                                            */
/*                                                                           */
/*      Parameters:     F_TYPE x[]      = phase data                         */
/*                                        x[0] = # data phase points         */
/*                                        x[1] = analysis start              */
/*                                        x[2] = analysis end                */
/*                      F_TYPE *p_sig   = pointer to new Theo1 deviation     */
/*                      F_TYPE tau      = data averaging time = tau_0        */
/*                      int m           = averaging factor = tau/tau_0       */
/*						float *fBias	= pointer to bias factor             */
/*						int *nNoise		= pointer to noise type (alpha)      */
/*						BOOL *bAbort    = pointer to abort flag              */                                                                     
/*						BOOL bProgress  = progress indicator flag            */
/*						BOOL bCalcBias  = bias calc flag                     */
/*						void *ProgressFunction()                             */
/*									    = pointer to progress indicator      */
/*									 	  funct with args int nPercentDone   */
/*										  and char *szMessage                */
/*                                                                           */
/*      Return:         float           = # analysis points,                 */
/*                                        or -1 if error                     */
/*                                                                           */
/*      Notes:          Embedded zero phase values treated as gaps in data.  */
/*                      The analysis data consists of n phase samples at     */
/*                      interval tau_0.  Denominator term (n-m) must be >0.  */
/*                      This sets max m=n-1.  m and tau must be >0, and m    */
/*                      must be even.  The Tho1 bias correction is the      */
/*						average ratio of Avar to Tho1 dev over the range    */
/*						of averaging factors where they both exist.          */
/*                                                                           */
/*                      Calls AutoTheo1Bias() to calc bias correction.       */
/*						Need to calc bias only first time during a run.      */
/*						NewTheo1BiasToAlpha() is then used to determine      */
/*						noise type from bias factor.                         */
/*                                                                           */
/*                      This function NewTheo1() calls original Theo1()      */
/*                      It optionally does bias removal if bCalcBias flag    */
/*                      is set.                                              */
/*                                                                           */
/*      Refs:           1. D.A. Howe and T.K. Peppler, "Estimation of Very   */
/*                         Long Term Frequency Stability Using a Special     */
/*                         Purpose Statistic", IEEE Int'l FCS 2003.          */
/*                                                                           */
/*                      2. T.N Tasset and D.A. Howe, "A Practical Tho1      */
/*                         Algorithm", Private e-mail communication,         */
/*						   October 31, 2003.                                 */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/21/03    Created                                              */
/*			12/24/03	Added *fBias and *nNoise arguments                   */
/*			12/31/03	Added bCalcBias argument                             */
/*			12/31/06	Changed call to NewTheo1BiasToAlpha() - No "New"     */
/*						Added to FrequenC DLL Ver 3.0 as NT1.C               */
/*                                                                           */
/* (c) Copyright 2003-6  Hamilton Technical Services  All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float __declspec(dllexport) FAR PASCAL NewTheo1(F_TYPE x[], F_TYPE *p_sig,
	F_TYPE tau, int m, float *fBias, int *nNoise, BOOL *bAbort,
	BOOL bProgress, BOOL bCalcBias,
	void (*ProgressFunction)(int nPercentDone, char *szMessage))
{
	// Local variables
	int n;                              // # phase data points for analysis
	float fCount;						// # non-gap analysis points

	// Set n to # analysis points
	n=(int)(x[2]-x[1]+1);
    
	// Check for NG arguments
	if( (m<1) || (n<m) || (tau<=0) || (m%2) )
    {
        return -1.0;                                               /* error */
    }

	// Calculate Tho1
	// Parameters are simply passed through
	fCount=Theo1(x, p_sig, tau, m, bAbort, bProgress, ProgressFunction);

	// Bail out if abort
	if(*bAbort)
	{
		return -1.0;
	}

	// Error trap for bad calc
	if(fCount==-1.0)
	{
		return -1.0;
	}

	// Calc bias if requested
	// Otherwise use value passed to function
	if(bCalcBias)
	{
		// Find average bias factor for Tho1 variance
		*fBias=AutoTheo1Bias(x, tau);
	}

	// Find estimated noise type
	#if(0) // Obsolete code 12/31/06
	*nNoise=NewTheo1BiasToAlpha(*fBias, m);
	#endif

	#if(1) // New code 12/31/06
	*nNoise=Theo1BiasToAlpha(*fBias, m);
	#endif

	// Remove bias
	*p_sig*=sqrt(*fBias);
    
	// Return result or error code
    if(fCount>0)                                     
    {
        return fCount;                          /* # non-gap analysis points */
    }
    else
    {
        return -1.0;                                                /* error */
    }
}                                                          /* end NewTheo1() */

/*****************************************************************************/