/*****************************************************************************/
/*                                                                           */
/*                          SpectrumCalc()                                   */
/*                                                                           */
/*      Function to calculate & plot power spectrum                          */
/*      Uses FFT functions from Science & Engineering Tools                  */
/*      # data points must be a power of 2                                   */
/*      Data must not have any gaps                                          */
/*                                                                           */
/*      Function calls from Quinn-Curtis S&E Library:                        */
/*          WindowFFTData()                                                  */
/*          PowerSpectrumCalc()                                              */
/*                                                                           */
/*      Parameters:                                                          */
/*          y[]         = time-domain data; y[0] = *y = # data points        */
/*          xdata[]     = real data points -> PSD results                    */
/*          ydata[]     = imaginary data points (0) -> Fourier freq results  */
/*          tau         = time interval of data samples                      */
/*          n           = # analysis data points                             */
/*          win_num     = # windowings                                       */
/*        2*win_type    = windowing method code:                             */
/*                  0   = Rectangular             win_type = 0               */
/*                  1   = Parzen (not used)                                  */
/*                  2   = Hanning                            1               */
/*                  3   = Welch (not used)                                   */
/*                  4   = Hamming                            2               */
/*                  5   = Blackman (not used)                                */
/*          phase_noise_type:  0=Sx(f), 1=So(f)    , 2=L(f)                  */
/*          correction_factor: 0=1    , 1=(2**v) , 2=(*v) & x10          */
/*          af          = averaging factor (int power of 2)                  */
/*			data_type   = time-domain data type: 0=phase, 1=freq             */
/*      Return          = # FFT frequencies                                  */
/*                                                                           */
/*      Revision record:                                                     */
/*          12/26/96    Adapted from spectrum_calc() of Stable/DOS           */
/*          04/12/97    Added code to preserve original data                 */
/*          04/01/02    Added af parameter & trial code for averaging        */
/*          07/11/02    Added PSD data saving to disk                        */
/*          07/27/02    Added zero padding                                   */
/*          07/30/02    Revised to handle analysis limits                    */
/*			03/20/03	Adapted to FrequenC DLL                              */
/*						Added data_type argument and return                  */
/*                                                                           */
/*****************************************************************************/

// FrequenC header file
#include "frequenc.h"

// Local defines
#define realtype float
#ifndef RECTANG
#define RECTANG 0
#define PARZEN  1
#define HANNING 2
#define WELCH   3
#define HAMMING 4
#define EXACTB  5
#endif

// FFT Windowing Types
#define RECTANGULAR_WIN 0
#define HANNING_WIN     1
#define HAMMING_WIN     2
#define MULTITAPER_WIN  3

// Local function prototypes - not exported
double RemoveMean(float *x, int lx);
void WindowData(realtype *x,int numdat,int window);
void WindowFFTData(realtype *xreal,realtype *yimag, int numdat,int window);
void fft(realtype *xreal, realtype *yimag, int numdat, int flag);
void PowerSpectrumCalc(realtype *xreal, realtype *yimag,int numdat,realtype delta);
void fft2 (realtype *aa, realtype *bb, int n, int m, int ks);
void reorder (realtype *aa, realtype *bb, int n, int m, int ks, int reel);

// SpectrumCalc() top-level function
int __declspec(dllexport) FAR PASCAL SpectrumCalc(F_TYPE y[], float xdata[],
	float ydata[], float tau, int n, int win_num, int win_type,
	int phase_noise_type, float correction_factor, int af, int data_type)
{
    int i;                          // General index
    int j;                          // Index for # windowings
    int k;                          // Index for averaging 
    static int offset;              // Index of first analysis data point
    static int nn;                  // Padded array size
    static int nNum;                // # data points in input array
    static int nFFT;                // # data points in one FFT section
    static float pad_factor;        // Correction factor for zero padding
    static float correction[3][4] =
    {
         { (float)0.00, (float)0.000, (float)0.000, (float)0.000 },
            // Rectangular
         { (float)0.00, (float)0.426, (float)0.563, (float)0.647 },
            // Hanning
         { (float)0.00, (float)0.401, (float)0.542, (float)0.627 }
            // Hamming
    };

   // These are log power amplitude corrections factors for wideband noise
   // 1st index is win_type; 2nd index is win_num
   static float bw_correction;
   static float log_mult;
   static F_TYPE avg;

    // The xdata array gets loaded with the time domain data.
    // This array is allocated with space for the # points & analysis limits.
    // These are put into the xdata array initially to allow it to use the
    // FillFloatGaps() function.
    // Then the header info is removed from the xdata array for the FFT calcs.

   // Input parameter n the # of analysis points - not necessarily a power-of-2

   // Find data offset
    offset=(int)y[1]+2; // Index of first data point to be analyzed

   // Copy analysis limits info into xdata[]
    xdata[0]=(float)n;
    xdata[1]=(float)1;
    xdata[2]=(float)n;

    // Load analysis data into the xdata array with header info
    for(i=0; i<n; i++)
    {
        xdata[i+ARRAY_OFFSET]=(float)y[i+offset];
    }

    // Make sure no there are gaps in the analysis data
    FillFloatGaps(xdata, data_type);

    // Eliminate header from the analysis data
    for(i=0; i<n; i++)
    {
        xdata[i]=xdata[ARRAY_OFFSET+i];                      /* real data points */
        ydata[i]=0.0;                               /* imaginary data points */
    }

    // Remove DC component
    RemoveMean(xdata, n);

    // The original FFT code just analyzes the entire data set as one block
    // w/o averaging.  Thus a longer record at the same tau increases the
    // frequency resolution but does not reduce the variance of the FFT
    // results.  To do that, one must break the data into sections, analyze
    // each section separately, and then average them to produce better FFT
    // values.  Doing so obviously shortens each individual data record, so
    // the FFT results won't extend to as low a Fourier frequency.  The variance
    // of the FFT values is thereby reduced from 100% to 100% divided by the
    // sqrt of the # averages, which should be an integer power of 2.
    // The averaged FFT can be done in the same pair of xdata and ydata arrays.
    // The xdata array starts with the real data points, and is filled with the
    // PSD results.  The ydata array starts with 0's and gets the Fourier freqs.
    // For averaging, just FFT each section separately in-place.  Then sum the
    // PSDs and Fourier freqs in the 1st section, and divide them by # averages.
    // Then continue to apply BW correftion and log mult to 1st section.
    // Both nNum and nAF are integer powers of 2, so k is also
    // e.g. if n is 512 and nAF is 8 then k goes from 0 to 7
    // and each section of FFT has nFFT=64 points.

    // Find size of next higher power-of-two = allocated size
    nn=(int)pow(2.0, (double)(ceil(log(n)/log(2))));

    // Zero the padded portion of the data
    for(i=n; i<nn; i++)
    {
        xdata[i]=0.0;                                    /* real data points */
        ydata[i]=0.0;                               /* imaginary data points */
    }

    // Zero padding correction factor
    // Square of ratio of total to actual (non-zero) FFT points
    pad_factor=((float)(nn)*(float)(nn))/((float)(n)*(float)(n));

    // Calc the # of averaged FFT points
    nFFT=nn/af;

    // Perform FFT on each section
    for(k=0; k<af; k++)
    {
        // Apply the desired windowing function
        j=win_num;
        while(j--)
        {
            WindowFFTData(&xdata[k*nFFT], &ydata[k*nFFT], nFFT, win_type);
                                                     /* window win_num times */
        }

        // Calculate the power spectrum using S&E function
        PowerSpectrumCalc(&xdata[k*nFFT], &ydata[k*nFFT], nFFT, tau);
                                                       /* calc power spectrum */
    }

    // Sum the results into the 1st section
    // Note that summation starts a k=1 (not k=0) - don't add 1st section to itself
    for(k=1; k<af; k++)
    {
        for(i=0; i<nFFT; i++)
        {
            xdata[i]+=xdata[k*nFFT+i];
            ydata[i]+=ydata[k*nFFT+i];
        }
    }
    
    // Find the average values in 1st section
    for(i=0; i<nFFT; i++)
    {
        xdata[i]/=af;
        ydata[i]/=af;
    }

    // Calc bandwidth correction
    bw_correction=(float)log10(ydata[1]);

    // Apply zero padding correction
    correction_factor*=pad_factor;

    // Set log multiplier factor
    if(phase_noise_type==2)                                          /* L(f) */
    {
        log_mult=10;
    }
    else                                                    /* Sx(f) & So(f) */
    {
        log_mult=1;
    }

    // Drop dc term from PSD and convert spectral data to log form
    // Note that xdata is PSD & ydata is Fourier freq
    for(i=0; i<(nFFT/2)-1; i++)
    {
        ydata[i]=ydata[i+1];                     /* Fourier frequency values */
        if(xdata[i+1]>0)
        {
            xdata[i]=(float)(log_mult*(log10(xdata[i+1]*correction_factor)
                     - bw_correction + correction[win_type/2][win_num]));
                            /* power spectrum values plus correction factors */
        }
        else
        {
            // Error in power spectrum calc
            break;
        }
    }

    // Save PSD data - Must move to Stable32 calling function
    // PSDWrite(szPSDFile, ydata, xdata, (nFFT/2)-1);

	return (nFFT/2)-1;
}

/*****************************************************************************/
/*                                                                           */
/*      Quinn-Curtis Science & Engineering Library FFT Functions             */
/*                                                                           */
/*      TC-006 S/N 18146 Rev 8.01 Date: 6/1/92 File: FFT.C (Excerpt)         */
/*                                                                           */
/*      Functions:                                                           */
/*      Parzen()    Hanning()       Hamming()         ExactBlackman()        */
/*      Welch()     WindowData()    WindowFFTData()   fft()                  */
/*      fft2()      reorder()       SumAnd Square()   PowerSpectrumCalc()    */
/*                                                                           */
/*      Note: The Stable/Win usage of these functions is with realtype       */
/*      defined as a float.  The local header fft_.h is a special version    */
/*      of fft.h that calls in reeltype.h which defines realtype as float    */
/*      and has prototypes for these functions.                              */
/*                                                                           */
/*      Copyright (c) Quinn-Curtis, 1992                                     */
/*                                                                           */
/*****************************************************************************/

// File macros
#ifndef M_PI // Defined in math.h under Borland C++ but not Visual C++
#define M_PI        3.14159265358979323846
#endif

/* Apply a parzen window */
realtype Parzen (realtype n, realtype j)
{
    realtype fret;

    fret = (realtype) (1.0 - fabs((j - 0.5 * (n - 1.0)) / (0.5 * (n - 1.0))));
    return(fret);
}

/* Apply a hanning window */
realtype Hanning (realtype n, realtype j)
{
    realtype fret;

    fret =  (realtype) (0.5 * (1.0 - cos(2.0 * M_PI * j / (n-1.0))));
    return(fret);
}

/* Apply a hamming window */
realtype Hamming (realtype n, realtype j)
{
    realtype fret;

    fret =  (realtype) (0.54 - 0.46 * cos(2.0 * M_PI * j / (n-1.0)));
    return(fret);
}

/* Apply a blackman window */
realtype ExactBlackman (realtype n, realtype j)
{
    realtype fret;

    fret =  (realtype) (0.42 - 0.50 * cos(2.0 * M_PI * j /(n-1.0)) +
            0.08 * cos(4.0 * M_PI * j / (n-1.0)));
    return(fret);
}

/* Apply a welch window */
realtype Welch (realtype n, realtype j)
{
    realtype fret;
    fret =  (realtype) ((j - 0.5 * (n - 1.0)) / (0.5 * (n + 1.0)));
    fret = (realtype) (1.0 - fret*fret);
    return(fret);
}

/* Apply a window for an array */
void WindowData (realtype *x, int numdat, int window)
{
   int i;
   realtype multiplier;

   for ( i = 0; i < numdat; i++ )
   {
      switch ( window )
      {
         case RECTANG:
            multiplier = (realtype) 1.0;
            break;
         case PARZEN:
            multiplier = Parzen ((realtype)numdat, (realtype)i);
            break;
         case HANNING:
            multiplier = Hanning ((realtype)numdat, (realtype)i);
            break;
         case WELCH:
            multiplier = Welch ((realtype)numdat, (realtype)i);
            break;
         case HAMMING:
            multiplier = Hamming ((realtype)numdat, (realtype)i);
            break;
         case EXACTB:
            multiplier = ExactBlackman ((realtype)numdat, (realtype)i);
            break;
         default:
            multiplier =  (realtype) 1.0;
            break;
      }
      x[i] = multiplier * x[i];
   }
}

/* Apply a window for both x and y data sets */
void WindowFFTData (realtype *xdata, realtype *ydata, int numdat, int window)
{
    WindowData (xdata, numdat, window);
    WindowData (ydata, numdat, window );
}

/* Complex Fourier Transform */
/* number of points n must be a power of 2 */
void fft (realtype *xr, realtype *yi, int n, int inverse)
{
    int nn, j;
    int      m;
    realtype pp, qq;

    m = -1;
    nn = n;

    /* determine power of 2 */
    while (nn > 0)
    {
        nn = nn >> 1;
        m++;
    }

    fft2 (xr, yi, n, m, n);
    reorder (xr, yi, n, m, n, 0);
    if (inverse)
    {
        pp = (realtype) 1.0 / n;
        qq = -pp;
        for (j = 0; j < n; j++)
        {
            xr [j] *= pp;
            yi [j] *= qq;
        }
    }
    else
    {
        for (j = 0; j < n; j++)
            yi[j] = -yi[j];
    }
}

/* FFT for one variable of dimension n = 2^m */
void fft2 (realtype *x, realtype *y, int n, int m, int ks)
{
    int        indx, k0, k1, k2, k3, span, j, k, kb, kn, mm, mk;
    realtype    rad, c1, c2, c3, s1, s2, s3, ck, sk, sq;
    realtype    x0, x1, x2, x3, y0, y1, y2, y3;
    int *cc;

    cc = (int *) calloc (m + 1, sizeof (int));

    sq = (realtype) 0.707106781187;
    sk = (realtype) 0.382683432366;
    ck = (realtype) 0.92387953251;

    cc[m] = ks;
    mm = (m / 2) * 2;
    kn = 0;
    for (k = m - 1; k >= 0; k--)
        cc[k] = cc[k+1] / 2;
    rad = (realtype) 6.28318530718 / (cc[0] * ks);
    mk = m - 5;

    do
    {
        kb = kn;
        kn = kn + ks;

        if (mm != m)
        {
            k2 = kn;
            k0 = cc[mm] + kb;
            do
            {
                k2--;
                k0--;
                x0 = x [k2];
                y0 = y [k2];
                x[k2] = x[k0] - x0;
                x[k0] = x[k0] + x0;
                y[k2] = y[k0] - y0;
                y[k0] = y[k0] + y0;
            }  while (k0 > kb);
        }

        c1 = (realtype) 1.0, s1 = (realtype) 0.0;
        indx = 0;
        k = mm - 2;
        j = 3;

        if (k >= 0)
            goto l2;
        else
            if (kn < n)
                continue;
            else
                break;

l1:     while (1)
        {
            if (cc[j] <= indx)
            {
                indx = indx - cc[j];
                j--;
                if (cc[j] <= indx)
                {
                    indx = indx - cc[j];
                    j--;
                    k += 2;
                }
            }
            else
                break;
        }

        indx = cc[j] + indx;
        j = 3;

l2:     span = cc[k];
        if (indx != 0)
        {
            c2 = indx * span * rad;
            c1 = (realtype) cos (c2);
            s1 = (realtype) sin (c2);
l5:
            c2 = c1 * c1 - s1 * s1;
            s2 = (realtype) 2.0 * s1 * c1;
            c3 = c2 * c1 - s2 * s1;
            s3 = c2 * s1 + s2* c1;
        }
        for (k0 = kb + span - 1; k0 >= kb; k0--)
        {
            k1 = k0 + span;
            k2 = k1 + span;
            k3 = k2 + span;
            x0 = x[k0];
            y0 = y [k0];
            if (s1 == 0)
            {
                x1 = x[k1]; y1 = y [k1];
                x2 = x[k2]; y2 = y [k2];
                x3 = x[k3]; y3 = y [k3];
            }
            else
            {
                x1 = x[k1] * c1 - y [k1] * s1;
                y1 = x[k1] * s1 + y [k1] * c1;
                x2 = x[k2] * c2 - y [k2] * s2;
                y2 = x[k2] * s2 + y [k2] * c2;
                x3 = x[k3] * c3 - y [k3] * s3;
                y3 = x[k3] * s3 + y [k3] * c3;
            }
            x [k0] = x0 + x2 + x1 + x3; y[k0] = y0 + y2 + y1 + y3;
            x [k1] = x0 + x2 - x1 - x3; y[k1] = y0 + y2 - y1 - y3;
            x [k2] = x0 - x2 - y1 + y3; y[k2] = y0 - y2 + x1 - x3;
            x [k3] = x0 - x2 + y1 - y3; y[k3] = y0 - y2 - x1 + x3;
        }
        if (k > 0)
        {
            k = k - 2;
            goto l2;
        }
        kb = k3 + span;
        if (kb < kn)
        {
            if (j == 0)
            {
                k = 2;
                j = mk;
                goto l1;
            }
            j--;
            c2 = c1;
            if (j == 1)
            {
                c1 = c1 * ck + s1 * sk;
                s1 = s1 * ck - c2 * sk;
            }
            else
            {
                c1 = (c1 - s1) * sq;
                s1 = (s1 + c2) * sq;
            }
            goto    l5;
        }
    }
    while (kn < n);

    free (cc);
}

void reorder ( realtype *x, realtype *y, int n, int m, int ks, int reel)
{
    int i, j, indx, k, kk, kb, k2, ku, lim, p;
    realtype  temp;

    int *cc, *lst;

    cc = (int *) calloc (m + 1, sizeof (int));
    lst = (int *) calloc (m + 1, sizeof (int));

    cc [m] = ks;
    for (k = m; k > 0; k--)
        cc [k-1] = cc [k] / 2;
    p = j = m - 1;
    i = kb = 0;
    if (reel)
    {
        ku = n - 2;
        for (k = 0; k <= ku; k += 2)
        {
            temp = x [k + 1];
            x [k + 1] = y [k];
            y [k] = temp;
        }
    }
    else
        m--;
    lim = (m + 2) / 2;
    if (p > 0)
    do
    {
        ku = k2 = cc [j] + kb;
        indx = cc [m - j];
        kk = kb + indx;

        do
        {
             k = kk + indx;
             do
             {
                 temp = x [kk];
                  x [kk] = x [k2];
                  x [k2] = temp;
                  temp = y [kk];
                  y [kk] = y [k2];
                  y [k2] = temp;
                  kk++;
                  k2++;
             }
             while (kk < k);

             kk = kk + indx;
             k2 = k2 + indx;
        }
        while (kk < ku);

        if (j > lim)
        {
            j--, i++;
            lst [i] = j;
            continue;
        }
        kb = k2;
        if (i > 0)
        {
            j = lst [i];
            i--;
            continue;
        }
        if (kb < n)
            j = p;
        else
            break;
    }
    while (1);

    free (cc);
    free (lst);
}

realtype SquareAndSum (realtype a, realtype b)
{
   realtype fret;

   fret = a * a + b * b;
   return(fret);
}

/* Calculate the power spectrum periodogram of a sampled dataset */
void PowerSpectrumCalc(realtype *xreal, realtype *yimag,
    int numdat, realtype delta)
{
    realtype timespan, normal, n;
    int i;

    n = (float)numdat;
    normal =  (realtype) 1.0 / (n * n);
    fft (xreal, yimag, numdat, 0);
    xreal[0] = SquareAndSum (xreal[0], yimag[0]) * normal;

    for (i = 1; i <= (numdat / 2) - 1; i++)
    {
        xreal[i] = normal * (SquareAndSum(xreal[i], yimag[i]) +
        SquareAndSum (xreal[numdat - i], yimag[numdat - i]));
    }
    i = numdat / 2;
    xreal[i] = SquareAndSum(xreal[i], yimag[i]) * normal;
    timespan = (realtype) ((1.0*numdat) * delta);
    for (i = 0; i <= (numdat/2); i++)
    {
        yimag[i] = (realtype) (1.0*i) / timespan;
    }
}

/* Remove mean from data array */
/* Cloned from same code used in multitaper PSD calc */
double RemoveMean(float *x, int lx)
{
    int k;
    double mean;

    mean = 0.;
    if(lx<2)
    {
        return mean;
    }

    for(k=0; k<=lx; k++)
    {
        mean= x[k]+mean;
    }

    mean=mean/(float)lx;

    for(k=0; k<=lx; k++)
    {
        x[k]=(float)(x[k]-mean);
    }

    return mean;
}

/*****************************************************************************/

