/*****************************************************************************/
/*                                                                           */
/*                           TotalvarBias()                                  */
/*                                                                           */
/*      Function to calculate the TOTALVAR bias function                     */
/*                                                                           */
/*      Parameters:     int nAlpha    	 = noise type (-2 to +2)             */
/*						F_TYPE fT        = record length                     */
/*						F_TYPE fTau		 = analysis averaging time           */
/*										 = meas interval * avg factor        */
/*																			 */
/*		Note:		    Record length    = nNum * measurement interval       */
/*						where nNum       = # phase data points               */
/*                                                                           */
/*      Return:         float          	 = TOTALVAR bias factor              */
/*                                         or -1 if error                    */
/*                                                                           */
/*      The TOTALVAR bias factor is the ratio of the expected value of       */
/*		TOTALVAR to AVAR.  To correct a calculated TOTALDEV value for a      */
/*		certain noise type, record length & analysis tau, divide it by the   */
/*		square root of the bias factor returned by this function.            */
/*                                                                           */
/*		References:		(1) D.A. Howe, "Method of Improving the Estimation   */
/*							of Long-Term Frequency Variance", Proc. 11th     */
/*							EFTF, March, 1997, (to be published).            */
/*						(2) Private e-mail communication from C.A.           */
/*                          Greenhall/JPL via D.A. Howe/NIST 03/28/97.       */
/*                                                                           */
/*                      TOTALVAR Bias =  1 - c * nTau / (nT - nTau)          */
/*                      where c = 1/(6 ln 2) for F  FM noise (nAlpha=-1)     */
/*                                3/8        for RW FM noise (nAlpha=-2)     */
/*                                                                           */
/*      Revision record:                                                     */
/*          04/05/97    Created                                              */
/*			01/01/98	Changes for MS VC++ compatibility & warnings         */
/*                                                                           */
/* (c) Copyright 1997-8 Hamilton Technical Services   All Rights Reserved    */
/*                                                                           */
/*****************************************************************************/

#include "frequenc.h"

float __declspec(dllexport) FAR PASCAL TotalvarBias(int nAlpha, F_TYPE fT,
	F_TYPE fTau)
{
	// Local variables
	float fC;           // Correction constant
	float fBias;		// Bias factor

	#if(0) // For testing
	TRACE_INT(nAlpha);
	TRACE_EXP(fT);
	TRACE_EXP(fTau);
	#endif

	// Sort by noise type
	switch(nAlpha)
	{
		case -2: // RW FM
		{
			fC=3.0/8.0;
			fBias=(float)(1.0-fC*fTau/(fT-fTau));
		}
		break;

		case -1: // F FM
		{
			fC=(float)(1.0/(6.0*log(2.0)));
			fBias=(float)(1.0-fC*fTau/(fT-fTau));
		}
		break;

		case -0: // W FM
		{
			fBias=1.0;
		}
		break;

		case 1: // F PM
		{
			fBias=1.0;
		}
		break;

		case 2: // W PM
		{
			fBias=1.0;
		}
		break;

		default: // Undefined
		{
			fBias=-1.0; // Error
		}
		break;
	}

	#if(0) // For testing
	TRACE_EXP(fBias);
	#endif

	return (fBias);
}

/*****************************************************************************/

